module SPC_Tests (tests) where

import Control.Concurrent (threadDelay)
import Control.Monad (forM, forM_, replicateM)
import Data.IORef
import SPC
import GenServer
import Test.Tasty (TestTree, localOption, mkTimeout, testGroup)
import Test.Tasty.HUnit (assertFailure, testCase, (@?=))
import Data.Either (isRight)

tests :: TestTree
tests =
  localOption (mkTimeout 3000000) $
    testGroup
      "SPC (core)"
      [
        testCase "worker-add" $ do
          spc <- startSPC
          w <- workerAdd spc "Nikolaj"
          isRight w @?= True,
        testCase "worker-add-2" $ do
          spc <- startSPC
          _ <- workerAdd spc "Nikolaj"
          w <- workerAdd spc "Alba"
          isRight w @?= True,
        testCase "worker-add-3" $ do
          spc <- startSPC
          _ <- workerAdd spc "Nikolaj"
          _ <- workerAdd spc "Alba"
          w <- workerAdd spc "Sebastian"
          isRight w @?= True,
        testCase "worker-add-2-fail" $ do
          spc <- startSPC
          _ <- workerAdd spc "Nikolaj"
          w <- workerAdd spc "Nikolaj"
          isRight w @?= False,
        testCase "worker-job" $ do
          spc <- startSPC
          w <- workerAdd spc "Nikolaj"
          ref <- newIORef False
          case spc of (SPC c) -> sendTo c (writeIORef ref True) 1
          isRight w @?= True
          v <- readIORef ref
          v @?= True
      ]
